#!/usr/bin/env python
# Mutation-based fuzzer

import random
import getopt
import sys

# Some mutations

# Trim the last characters of an input
def trim(s):
    max_len = random.randint(0, len(s) - 1)
    sys.stderr.write("Trimming at " + repr(max_len) + "\n")
    return s[:max_len]

# Flip a random bit at a random position
def flip(s):
    pos = random.randint(0, len(s) - 1)    
    # Avoid flipping Bit 7 - breaks ASCII encoding
    bit = random.randint(0, 6)
    sys.stderr.write("Flipping bit " + repr(bit) + " at " + repr(pos) + "\n")
    flipped = chr(ord(s[pos]) ^ (1 << bit))
    return s[:pos] + flipped + s[pos + 1:]
    
# Swap two bytes at a random position
def swap(s):
    pos = random.randint(0, len(s) - 2)
    sys.stderr.write("Swapping at " + repr(pos) + "\n")
    return s[:pos] + s[pos + 1] + s[pos] + s[pos + 2:]

def help():
    name = sys.argv[0]  # Program name
    sys.stderr.write("Usage: " + name + " [-h] [INPUT...]\n")
    sys.exit(0)
    
def process(f):
    # Read specified file
    s = f.read()
    
    # Choose a mutator (randomly)...
    mutations = [trim, flip, swap]    
    mutator = random.choice(mutations)
    
    # ...and output its result.
    mutated = mutator(s)
    sys.stdout.write(mutated)

# Main function
def main():
    opts, args = getopt.getopt(sys.argv[1:], "h")
    for opt, a in opts:
        if opt == "-h":
            help()
        else:
            assert False, "unhandled option"
            
    if len(args) == 0:
        # No arguments: read standard input
        process(sys.stdin)
        sys.exit(0)
        
    # Process one argument after another
    for arg in args:
        # Read specified file
        process(open(arg))

if __name__ == "__main__":
    main()
